#include "AlbumPanel.h"

#include "BCell.h"

@interface newDirPanel: NSPanel
{
  id thePath;
  id theTextField;
  id theOKButton;
  id theCancelButton;
}

- (id) init;
- (void)dealloc;
- (int)runModalForDirectory: (NSString *)aPath;
- (void)OKButtonAction: (id)sender;
- (void)CancelButtonAction: (id)sender;

@end

@implementation newDirPanel

- (id) init
{
  [super initWithContentRect: NSMakeRect(200, 200, 272, 63)
	 styleMask: (NSTitledWindowMask)
	 backing: NSBackingStoreBuffered
	 defer: YES];

  [self setTitle: @"New directory"];  

  theTextField = [[NSTextField alloc] initWithFrame: NSMakeRect(5, 34, 262, 24)];
  [theTextField setStringValue: @""];
  [[self contentView] addSubview: theTextField];

  theOKButton = [[NSButton alloc] initWithFrame: NSMakeRect(211, 5, 56, 24)];
  [theOKButton setStringValue: @"OK"];
  [theOKButton setTarget: self];
  [theOKButton setAction: @selector(OKButtonAction:)];
  [[self contentView] addSubview: theOKButton];

  theCancelButton = [[NSButton alloc] initWithFrame: NSMakeRect(150, 5, 56, 24)];
  [theCancelButton setStringValue: @"Cancel"];
  [theCancelButton setTarget: self];
  [theCancelButton setAction: @selector(CancelButtonAction:)];
  [[self contentView] addSubview: theCancelButton];

  return self;
}


- (void)dealloc
{
  RELEASE(thePath);
  RELEASE(theTextField);
  RELEASE(theOKButton);
  RELEASE(theCancelButton);
  [super dealloc];
}


- (int)runModalForDirectory: (NSString *)aPath
{
  int returnValue;

  thePath = [NSString stringWithString: aPath];

  [self  makeKeyAndOrderFront: self];
  returnValue = [NSApp runModalForWindow: self];
  [self orderOut: self];
  return returnValue;
}


- (void)OKButtonAction: (id)sender
{
  if ([[theTextField stringValue] length] > 0)
    {
      [[NSFileManager defaultManager] createDirectoryAtPath: [NSString stringWithFormat: @"%@/%@", thePath, [theTextField stringValue]] attributes: nil];
      [NSApp stopModalWithCode: OK_BUTTON];
    }
  else
    {
      [NSApp stopModalWithCode: CANCEL_BUTTON];
    }
}


- (void)CancelButtonAction: (id)sender
{
  [NSApp stopModalWithCode: CANCEL_BUTTON];
}

@end

@interface AlbumPanel(private)

- (void)OKButtonAction: (id)sender;
- (void)CancelButtonAction: (id)sender;
- (void)AddButtonAction: (id)sender;

/* mthodes delegate de theBrowser */
- (void)browser: (NSBrowser*)sender createRowsForColumn: (int)column inMatrix: (NSMatrix*)matrix;
- (void)browserAction: (NSBrowser *)sender;

@end

@implementation AlbumPanel

- (id)init
{
  [super initWithContentRect: NSMakeRect(200, 200, 272, 229)
	 styleMask: (NSTitledWindowMask |
		     NSResizableWindowMask)
	 backing: NSBackingStoreBuffered
	 defer: YES];

  [self setTitle: @"Album"];

  theAddButton = [[NSButton alloc] initWithFrame: NSMakeRect(5, 200, 24, 24)];
  [theAddButton setAutoresizingMask: (NSViewMaxXMargin | NSViewMinYMargin)];
  [theAddButton setStringValue: @"+"];
  [theAddButton setTarget: self];
  [theAddButton setAction: @selector(AddButtonAction:)];
  [[self contentView] addSubview: theAddButton];

  theBrowser = [[NSBrowser alloc] initWithFrame: NSMakeRect (5, 34, 262, 161)];
  [theBrowser setAutoresizingMask: NSViewWidthSizable | NSViewHeightSizable];
  [theBrowser setTitled: NO];
  [theBrowser setCellClass: [BCell class]];
  [theBrowser setAllowsMultipleSelection: NO];
  [theBrowser setDelegate: self];
  [[self contentView] addSubview: theBrowser];

  theOKButton = [[NSButton alloc] initWithFrame: NSMakeRect(211, 5, 56, 24)];
  [theOKButton setStringValue: @"OK"];
  [theOKButton setTarget: self];
  [theOKButton setAction: @selector(OKButtonAction:)];
  [theOKButton setAutoresizingMask: NSViewMinXMargin];
  [[self contentView] addSubview: theOKButton];

  theCancelButton = [[NSButton alloc] initWithFrame: NSMakeRect(150, 5, 56, 24)];
  [theCancelButton setStringValue: @"Cancel"];
  [theCancelButton setTarget: self];
  [theCancelButton setAction: @selector(CancelButtonAction:)];
  [theCancelButton setAutoresizingMask: NSViewMinXMargin];
  [[self contentView] addSubview: theCancelButton];

  return self;
}


- (void)dealloc
{
  RELEASE(theOKButton);
  RELEASE(theCancelButton);
  [super dealloc];
}


- (int)runModalForDirectory: (NSString *)aDir Subalbum: (NSString *)aSubalbum
{
  int returnValue;

  theAlbum = [NSString stringWithString: aSubalbum];
  theDirectory = [NSString stringWithString: aDir];

  [self  makeKeyAndOrderFront: self];
  returnValue = [NSApp runModalForWindow: self];
  [self orderOut: self];
  return returnValue;
}

- (void)OKButtonAction: (id)sender
{
  if ([theBrowser selectedCell] != nil)
    [NSApp stopModalWithCode: OK_BUTTON];
  else
    [NSApp stopModalWithCode: CANCEL_BUTTON];
}


- (void)CancelButtonAction: (id)sender
{
  [NSApp stopModalWithCode: CANCEL_BUTTON];
}


- (void)browser: (NSBrowser*)sender createRowsForColumn: (int)column inMatrix: (NSMatrix*)matrix
{
  NSString *path;
  NSAutoreleasePool *pool = [NSAutoreleasePool new];
  NSFileManager *manager = [NSFileManager defaultManager];
  NSArray *ar;
  int i, j;

  if (column == 0)
    {
      path = [NSString stringWithString: theDirectory];
    }
  else
    {
      path = [NSString stringWithString: [[theBrowser selectedCellInColumn: column-1] representedObject]];
    }

  ar = [manager directoryContentsAtPath: path];

  if (ar != nil)
    {
      j = 0;
      for (i=0; i<[ar count]; i++)
	{
	  NSBrowserCell *cell;
	  BOOL isDir;

	  [manager fileExistsAtPath: [NSString stringWithFormat: @"%@/%@", path, [ar objectAtIndex: i]] isDirectory: &isDir];

	  if ((isDir) && ([[ar objectAtIndex: i] characterAtIndex: 0] != '.'))
	    { 
	      [matrix addRow];
	      cell = [matrix cellAtRow: j column: 0];
	      [cell setStringValue: [ar objectAtIndex: i]];
	      [cell setRepresentedObject: [NSString stringWithFormat: @"%@/%@", path, [ar objectAtIndex: i]]];
	      [cell setLeaf: NO];
	      j++;
	    }
	}
    }
  
  RELEASE(pool);
}


- (void)browserAction: (NSBrowser *)sender
{

}


- (NSString *)selectedAlbum
{
  NSMutableString *res;
  NSArray *ar;
  NSArray *ar2;
  int count;
  int i;

  ar2 = [theDirectory pathComponents];
  count = [ar2 count];

  ar = [[[theBrowser selectedCell] representedObject] pathComponents];

  res = [NSMutableString stringWithString: @""];

  for (i=count; i<[ar count]; i++)
    {
      [res appendFormat: @"/%@", [ar objectAtIndex: i]];
    }

  [res deleteCharactersInRange: NSMakeRange(0, 1)];

  return res;
}


- (void)AddButtonAction: (id)sender
{
  newDirPanel *ndp;
  NSString *path;
  NSBrowserCell *cell;
  int res = CANCEL_BUTTON;

  ndp = [[newDirPanel alloc] init];
  
  cell = [theBrowser selectedCell];
  
  if (cell != nil)
    {
      path = [cell representedObject];
      
      if ([ndp runModalForDirectory: path] == OK_BUTTON)
	{
	  [theBrowser reloadColumn: 0];
	}
    }
  else
    {
      if ([ndp runModalForDirectory: theDirectory] == OK_BUTTON)
	{
	  [theBrowser reloadColumn: 0];
	}
    }
  
  //RELEASE(ndp);
}

@end
